import requests
import bs4
import random

from requests.auth import HTTPBasicAuth


#  Helper methods for BS4
def html_table_to_dict_list(table, only_text=False):
	items = []
	headings = [th.get_text().lower() for th in table.find("tr").find_all("th")]
	for row in table.find_all("tr")[1:]:
		item = [td for td in row.find_all("td")]
		if only_text:
			item = [i.get_text() for i in item]
		if len(item) > 1:
			items.append(dict(zip(headings, item)))
	return items


class NoJobsForUser(Exception):
	pass


class Scrapper(object):
	""" Main class for Doopla
	"""

	def __init__(self, web_ui_url, hadoop_user, user, passwd):
		super(Scrapper, self).__init__()
		self._auth = HTTPBasicAuth(user, passwd)
		self._hadoop_user = hadoop_user
		self._web_ui_url = web_ui_url
		print(self._web_ui_url)
		self._jobtracker_url = "{}/jobtracker.jsp".format(self._web_ui_url)
		self._jobdetail_template = "{}/jobdetails.jsp?jobid=".format(self._web_ui_url) + "{}"

	def fetch_html(self, url):
		r = requests.get(url, verify=False, auth=self._auth)
		soup = bs4.BeautifulSoup(r.text, 'html.parser')
		return soup

	def scrap_last_failed_job_id(self, ):
		"""Get's the last failed job id for the user"""
		html = self.fetch_html(self._jobtracker_url)
		table = html.find(id='failed_jobs').find_next_siblings()[0]
		jobs = []

		headings = [th.get_text().lower() for th in table.find("tr").find_all("td")]

		# Get the table data from Hadoop
		for row in table.find_all("tr")[1:]:
			job = [td.get_text() for td in row.find_all("td")]
			if len(job) > 1:
				job = job[:-2]
				jobs.append(dict(zip(headings, job)))

		target = None
		for j in jobs[::-1]:
			if j['user'] == self._hadoop_user:
				target = j['jobid']
				break

		return target

	def scrap_failure_output(self, jobid):
		# Get mapper failure if any

		def build_url(jobid, kind, cause):
			template = "{}/jobfailures.jsp?jobid={}&kind={}&cause={}"
			return template.format(self._web_ui_url, jobid, kind, cause)

		err_url_map = build_url(jobid, kind="map", cause="failed")
		err_url_red = build_url(jobid, kind="reduce", cause="failed")

		map_out_url = self.scrap_output_url(err_url_map)
		red_out_url = self.scrap_output_url(err_url_red)

		map_output = ("NOT FAILED MAPPPER TASKS :)", "")
		red_output = ("NOT FAILED REDUCER TASKS :)", "")

		if map_out_url:  # No failed tasks for mappers
			map_output = self.scrap_output_from_attempt(map_out_url)
		if red_out_url:
			red_output = self.scrap_output_from_attempt(red_out_url)

		return map_output, red_output

	def scrap_output_url(self, url):
		"""
		From the tables of failures for a particular job get
		one at random and obtain the url.
		returns the url to the output of that attempt
		"""
		html = self.fetch_html(url)
		table = html.find('table')

		if not table:
			return None

		items = html_table_to_dict_list(table)
		url = None

		no_items = len(items)

		if no_items > 0:
			i = random.randint(0, no_items - 1)
			url = items[i]['logs'].contents[0].attrs['href']
			print("Scrapping Job Attempt URL: {}".format(url))

		return url

	def scrap_output_from_attempt(self, url):
		""" returns the stdout and stderr output from the error url"""
		def clean_output(output):
			lines = output.split('\n')
			if len(lines) > 3:
				lines = lines[2:]
				return "\n".join(lines)
			return output

		html = self.fetch_html(url)
		output = html.find_all('pre')
		stderr, stdout = None, None
		print(url)
		if len(output) >= 2:
			stdout = clean_output(output[0].get_text())
			stderr = clean_output(output[1].get_text())

		return stdout, stderr

	def fetch_output(self, jobid=None):
		jobid = jobid or self.scrap_last_failed_job_id()

		if not jobid:
			raise NoJobsForUser()

		self.jobid = jobid

		print("Obtaining failing output for: {}".format(jobid))

		return self.scrap_failure_output(jobid)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
'''doopla

Usage:
doopla [<jobid>]
doopla -h | --help
doopla --version

Options:
-h --help       Show this screen.
--version       Show version.
'''

from __future__ import unicode_literals, print_function
from docopt import docopt
from doopla.scrapper import Scrapper, NoJobsForUser
from colorama import init
from colorama import Fore, Back, Style

from pygments import highlight
from pygments.lexers import PythonTracebackLexer
from pygments.formatters import TerminalFormatter

import ConfigParser
import os


__version__ = "0.2.1"
__author__ = "Miguel Cabrera"
__license__ = "MIT"


def read_config():
	config = ConfigParser.ConfigParser()
	loc = os.path.expanduser("~")
	try:
		with open(os.path.join(loc, ".doopla")) as source:
			config.readfp(source)
	except IOError:
		print("Cannot read configuration files. Exiting")
		exit(-1)

	return config


def print_output(string):
	if string:
		print(highlight(string, PythonTracebackLexer(), TerminalFormatter(bg='dark')))


def main():
	init()
	args = docopt(__doc__, version=__version__)

	config = read_config()

	user = config.get('main', "hadoop_user")
	http_user = config.get('main', "http_user")
	http_passwd = config.get('main', "http_password")
	webui_url = config.get('main', "webui_url")

	sc = Scrapper(webui_url, user, http_user, http_passwd)

	try:
		mapper, reducer = sc.fetch_output(args['<jobid>'])

		print("\nStdout for Jobid:" + Fore.BLUE + "  {} ".format(sc.jobid))
		print(Fore.RESET + Back.RESET + Style.RESET_ALL)

		print("Mapper::Stdout \n")
		print_output(mapper[0])
		print("Mapper::Stderr \n")
		print_output(mapper[1])

		print("Reducer::Stdout \n")
		print_output(reducer[0])
		print("Reducer::Stderr \n")
		print_output(reducer[1])

	except NoJobsForUser:
		print("No failed jobs for user {}".format(user))

if __name__ == '__main__':
	main()



# -*- coding: utf-8 -*-
import re
import sys
from setuptools import setup
from setuptools.command.test import test as TestCommand


REQUIRES = [
	'docopt',
	'requests>=1.2.7',
	'beautifulsoup4>=4.3.2',
	'pygments',
	'colorama'
]

class PyTest(TestCommand):
	def finalize_options(self):
		TestCommand.finalize_options(self)
		self.test_args = []
		self.test_suite = True

	def run_tests(self):
		import pytest
		errcode = pytest.main(self.test_args)
		sys.exit(errcode)


def find_version(fname):
	'''Attempts to find the version number in the file names fname.
	Raises RuntimeError if not found.
	'''
	version = ''
	with open(fname, 'r') as fp:
		reg = re.compile(r'__version__ = [\'"]([^\'"]*)[\'"]')
		for line in fp:
			m = reg.match(line)
			if m:
				version = m.group(1)
				break
	if not version:
		raise RuntimeError('Cannot find version information')
	return version

__version__ = find_version("command.py")


def read(fname):
	with open(fname) as fp:
		content = fp.read()
	return content

setup(
	name='doopla',
	version=__version__,
	description='A Python script to fetch the output of fail tasks from ours Hadoop clusters',
	long_description=read("README.md"),
	author='Miguel Cabrera',
	author_email='mfcabrera@gmail.com',
	url='https://github.com/trustyou/doopla',
	install_requires=REQUIRES,
	license=read("LICENSE"),
	zip_safe=False,
	keywords='doopla',
	packages=['doopla',],
	classifiers=[
		'Development Status :: 2 - Pre-Alpha',
		'Intended Audience :: Developers',
		'License :: OSI Approved :: MIT License',
		'Natural Language :: English',
		"Programming Language :: Python :: 2",
		'Programming Language :: Python :: 2.7',
		'Programming Language :: Python :: 3',
		'Programming Language :: Python :: 3.3',
		'Programming Language :: Python :: Implementation :: CPython',
		'Programming Language :: Python :: Implementation :: PyPy'
	],
	py_modules=['command'],
	entry_points={
		'console_scripts': [
			"doopla = command:main"
		]
	},
	tests_require=['pytest'],
	cmdclass={'test': PyTest}
)
